import eseries
import uuid
from dataclasses import dataclass
from sqlalchemy import create_engine
from typing import List
from typing import Optional
from sqlalchemy import ForeignKey
from sqlalchemy import String
from sqlalchemy.orm import DeclarativeBase, Session
from sqlalchemy.orm import Mapped
from sqlalchemy.orm import mapped_column
from sqlalchemy.orm import relationship


@dataclass
class TResistor:
    mpn: str
    symbol: str
    footprint: str
    description: str
    value: str
    tolerance: str
    rating: str


class Base(DeclarativeBase):
    pass


class Manufacturer(Base):
    __tablename__ = "manufacturers"

    id: Mapped[int] = mapped_column(primary_key=True)
    uuid: Mapped[str] = mapped_column(String(36))
    name: Mapped[str] = mapped_column(String(64))


class Resistor(Base):
    __tablename__ = "resistors"

    id: Mapped[int] = mapped_column(primary_key=True)
    uuid: Mapped[str] = mapped_column(String(36))
    name: Mapped[str] = mapped_column(String(64))
    manufacturer: Mapped[int] = mapped_column(ForeignKey("manufacturers.id", ondelete='CASCADE'))
    mpn: Mapped[str] = mapped_column(String(64))
    symbol: Mapped[str] = mapped_column(String(64))
    footprint: Mapped[str] = mapped_column(String(64))
    description: Mapped[str] = mapped_column(String(64))
    value: Mapped[str] = mapped_column(String(64))
    tolerance: Mapped[str] = mapped_column(String(16))
    rating: Mapped[str] = mapped_column(String(16))

    def __repr__(self) -> str:
        return f'Resistor(id={self.id!r}, uuid={self.uuid!r}, name={self.name!r})'


def get_resistor():
    for value in eseries.erange(eseries.E24, 1, 22_000_000):
        # print(f'{value=}')
        magnitude = max(0, len(str(int(value))) - 2)
        # print(f'{magnitude=}')
        base = 10 ** magnitude
        # print(f'base={base:_}')
        n_value = f'{value / base:g}'
        n_value_g = n_value
        # print(f'{n_value=}')
        suffix: str = ''
        suffix = 'k' if base >= 1_000 else suffix
        suffix = 'M' if base >= 1_000_000 else suffix

        if base >= 1_000_000:
            if not n_value.count('.'):
                n_value = ''.join([n_value, 'M'])
            else:
                n_value = n_value.replace('.', 'M')
        elif base >= 1_000:
            if not n_value.count('.'):
                n_value = ''.join([n_value, 'K'])
            else:
                n_value = n_value.replace('.', 'K')
        else:
            if not n_value.count('.'):
                n_value = ''.join([n_value, 'R'])
            else:
                n_value = n_value.replace('.', 'R')
        # print(f'RC0603JR-07{n_value}L')
        yield TResistor(
            mpn=f'RC0603JR-07{n_value}L',
            symbol='Device:R',
            footprint='Resistor_SMD:R_0603_1608Metric',
            description=f'RES {n_value_g}{suffix.upper()} OHM 5% 1/10W 0603',
            rating='0.1W',
            tolerance=u'±5%',
            value=f'{n_value_g}{suffix}Ω'
        )


def main() -> None:
    engine = create_engine("sqlite:///test.db", echo=True)
    Base.metadata.create_all(engine)

    with Session(engine) as session:
        yageo = Manufacturer(
            uuid=str(uuid.uuid4()),
            name='Yageo'
        )

        session.add(yageo)
        session.flush()
        session.refresh(yageo)

        for item in get_resistor():
            resistor_item = Resistor(
                name=item.mpn,
                uuid=str(uuid.uuid4()),
                manufacturer=yageo.id,
                mpn=item.mpn,
                symbol=item.symbol,
                footprint=item.footprint,
                description=item.description,
                value=item.value,
                tolerance=item.tolerance,
                rating=item.rating
            )

            session.add(resistor_item)

        session.commit()


if __name__ == '__main__':
    main()
