import uuid
from dataclasses import dataclass
from decimal import Decimal
from math import log10

import eseries
from sqlalchemy import Column, Integer, String, create_engine, ForeignKey, Text, Numeric
from sqlalchemy.orm import declarative_base, Session

Base = declarative_base()


class ModelManufacturer(Base):
    __tablename__ = 'manufacturers'

    id = Column(Integer, primary_key=True)
    uuid = Column(String(36))
    name = Column(Text)


class ModelResistor(Base):
    __tablename__ = 'resistors'

    id = Column(Integer, primary_key=True)
    uuid = Column(String(36))
    name = Column(Text)
    manufacturer = Column(ForeignKey('manufacturers.id', ondelete='CASCADE'))
    mpn = Column(Text)
    symbols = Column(Text)
    footprints = Column(Text)
    description = Column(Text)
    value = Column(Numeric)
    tolerance = Column(Text)
    rating = Column(Text)


@dataclass
class TResistor:
    mpn: str
    symbols: str
    footprints: str
    description: str
    value: str
    tolerance: str
    rating: str


class YageoResistorPartGenerator:
    @staticmethod
    def yageo_rc0603_e24_5percent():
        for value in eseries.erange(24, 1, 22_000_000):
            decade = int(log10(value))
            rank = 10**decade
            group = 10**(decade % 3)
            magnitude = Decimal(value) / Decimal(rank) * Decimal(group)
            False and print(f'{value=:_}, {decade=}, {rank=}, {group=:_}, {magnitude=}')

            value_class = ('R', '')
            if rank >= 1_000:
                value_class = ('K', 'k')
            if rank >= 1_000_000:
                value_class = ('M', 'M')

            generic_resistance_value = f'{float(magnitude):g}{value_class[1]}'
            yageo_resistance_value = f'{float(magnitude)}'.replace('.', value_class[0]).rstrip('0')
            mpn = f'RC0603JR-07{yageo_resistance_value}L'

            yield TResistor(
                mpn=mpn,
                symbols='Device:R',
                footprints='Resistor_SMD:R_0805_2012Metric;Resistor_SMD:R_0805_2012Metric_Pad1.20x1.40mm_HandSolder',
                description=f'RES {generic_resistance_value.upper()} OHM 5% 1/10W',
                value=f'{value}',
                tolerance='±5%',
                rating='0.1W'
            )


def main() -> None:
    engine = create_engine('sqlite:///test.db', echo=True)
    Base.metadata.create_all(bind=engine)

    session = Session(bind=engine)

    yageo = ModelManufacturer(name='Yageo', uuid=str(uuid.uuid4()))
    session.add(yageo)
    session.flush()
    session.refresh(yageo)

    print(f'{yageo.uuid=}')

    for part in YageoResistorPartGenerator.yageo_rc0603_e24_5percent():
        print(f'{part=}')
        resistor = ModelResistor(
            uuid=str(uuid.uuid4()),
            name=part.mpn,
            manufacturer=yageo.id,
            mpn=part.mpn,
            symbols=part.symbols,
            footprints=part.footprints,
            description=part.description,
            value=part.value,
            tolerance=part.tolerance,
            rating=part.rating
        )

        session.add(resistor)
    session.commit()


if __name__ == '__main__':
    main()
